@@ -35,7 +35,7 @@ mp_compare_mp_to_int(const MPNumber *x, int i)
 
 
 /* Convert x to radians */
-static void
+void
 convert_to_radians(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
     MPNumber t1, t2;
@@ -68,7 +68,7 @@ mp_get_pi(MPNumber *z)
 }
 
 
-static void
+void
 convert_from_radians(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
     MPNumber t1, t2;
@@ -154,11 +154,10 @@ mpsin1(const MPNumber *x, MPNumber *z, int do_sin)
 }
 
 
-void
-mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+static void
+mp_sin_real(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
-    int ie, xs;
-    float rx = 0.0;
+    int xs;
     MPNumber x_radians;
 
     /* sin(0) = 0 */
@@ -170,15 +169,10 @@ mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
     convert_to_radians(x, unit, &x_radians);
 
     xs = x_radians.sign;
-    ie = abs(x_radians.exponent);
-    if (ie <= 2)
-        rx = mp_cast_to_float(&x_radians);
-
     mp_abs(&x_radians, &x_radians);
 
     /* USE MPSIN1 IF ABS(X) <= 1 */
-    if (mp_compare_mp_to_int(&x_radians, 1) <= 0)
-    {
+    if (mp_compare_mp_to_int(&x_radians, 1) <= 0) {
         mpsin1(&x_radians, z, 1);
     }
     /* FIND ABS(X) MODULO 2PI */
@@ -226,21 +220,11 @@ mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
     }
 
     z->sign = xs;
-
-    /*  CHECK THAT ABSOLUTE ERROR LESS THAN 0.01 IF ABS(X) <= 100
-     *  (IF ABS(X) IS LARGE THEN SINGLE-PRECISION SIN INACCURATE)
-     */
-    if (ie <= 2 && fabs(rx) <= 100.0) {
-        float ry = mp_cast_to_float(z);
-        /*  THE FOLLOWING MESSAGE MAY INDICATE THAT B**(T-1) IS TOO SMALL. */
-        if (fabs(ry - sin(rx)) >= 0.01)
-            mperr("*** ERROR OCCURRED IN MPSIN, RESULT INCORRECT ***");
-    }
 }
 
 
-void
-mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+static void
+mp_cos_real(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
     /* cos(0) = 1 */
     if (mp_is_zero(x)) {
@@ -266,6 +250,55 @@ mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 }
 
 
+void
+mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+{
+    if (mp_is_complex(x)) {
+        MPNumber x_real, x_im, z_real, z_im, t;
+
+        mp_real_component(x, &x_real);
+        mp_imaginary_component(x, &x_im);
+
+        mp_sin_real(&x_real, unit, &z_real);
+        mp_cosh(&x_im, &t);
+        mp_multiply(&z_real, &t, &z_real);
+
+        mp_cos_real(&x_real, unit, &z_im);
+        mp_sinh(&x_im, &t);
+        mp_multiply(&z_im, &t, &z_im);
+
+        mp_set_from_complex(&z_real, &z_im, z);
+    }
+    else
+       mp_sin_real(x, unit, z);
+}
+
+
+void
+mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+{
+    if (mp_is_complex(x)) {
+        MPNumber x_real, x_im, z_real, z_im, t;
+
+        mp_real_component(x, &x_real);
+        mp_imaginary_component(x, &x_im);
+
+        mp_cos_real(&x_real, unit, &z_real);
+        mp_cosh(&x_im, &t);
+        mp_multiply(&z_real, &t, &z_real);
+
+        mp_sin_real(&x_real, unit, &z_im);
+        mp_sinh(&x_im, &t);
+        mp_multiply(&z_im, &t, &z_im);
+        mp_invert_sign(&z_im, &z_im);
+
+        mp_set_from_complex(&z_real, &z_im, z);
+    }
+    else
+       mp_cos_real(x, unit, z);
+}
+
+
 void
 mp_tan(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
